<?php

if (!defined('ABSPATH')) {
    exit();
}

class WMUHelper implements WMUConstants {

    private $optionsSerialized;
    private $maxFileSize;
    private $html5VideoTypes = array('.mp4' => 'video/mp4', '.webm' => 'video/webm', '.ogg' => 'video/ogg');
    private $html5AudioTypes = array('.mp3' => 'audio/mpeg', '.ogg' => 'audio/ogg', '.wav' => 'audio/wav');
    private $elementCount;
    public $imageSizes;
    public $imageAcceptExt;
    public $videoAcceptExt;
    public $fileAcceptExt;

    public function __construct($optionsSerialized) {
        $this->optionsSerialized = $optionsSerialized;
        $this->maxFileSize = $this->optionsSerialized->wmuMaxFileSize * 1024 * 1024;
        $this->initAllowedFileTypes();
        $this->imageSizes = $this->getImageSizes();
        $this->elementCount = $this->optionsSerialized->wmuIsMultiple ? $this->optionsSerialized->wmuMaxFileCount : 1;
    }

    private function initAllowedFileTypes() {
        $array = get_allowed_mime_types();
        if ($array && is_array($array)) {
            foreach ($array as $key => $value) {
                $subTypes = explode('|', $key);
                if ($subTypes && is_array($subTypes)) {
                    foreach ($subTypes as $type) {
                        if (preg_match('#^image#is', $value)) {
                            $this->optionsSerialized->wmuImageTypes[$type] = $value;
                            $this->imageAcceptExt[] = strtolower($type);
                        } else if (preg_match('#^video#is', $value) || preg_match('#^audio#is', $value)) {
                            $this->optionsSerialized->wmuVideoTypes[$type] = $value;
                            $this->videoAcceptExt[] = strtolower($type);
                        } else {
                            $this->optionsSerialized->wmuFileTypes[$type] = $value;
                            $this->fileAcceptExt[] = strtolower($type);
                        }
                    }
                } else {
                    if (preg_match('#^image#is', $value)) {
                        $this->optionsSerialized->wmuImageTypes[$key] = $value;
                        $this->imageAcceptExt[] = strtolower($key);
                    } else if (preg_match('#^video#is', $value) || preg_match('#^audio#is', $value)) {
                        $this->optionsSerialized->wmuVideoTypes[$key] = $value;
                        $this->videoAcceptExt[] = strtolower($key);
                    } else {
                        $this->optionsSerialized->wmuFileTypes[$key] = $value;
                        $this->fileAcceptExt[] = strtolower($key);
                    }
                }
            }
        }
    }

    public function getInputFilePattern($types) {
        $accept = '';
        foreach ($types as $key => $type) {
            if (strpos($key, '|')) {
                $subTypes = explode('|', $key);
                foreach ($subTypes as $subType) {
                    $accept .= '.' . $subType . ',';
                }
            } else {
                $accept .= '.' . $key . ',';
            }
        }
        return trim($accept, ',');
    }

    public function checkFiles($wmuMaxFileCount, $key, $typesArray) {
        $msgArray = array();
        if (count($_FILES[$key]['type']) > $wmuMaxFileCount) {
            $msgArray['code'] = 'msgMaxFileCount';
            wp_die(json_encode($msgArray));
        }

        for ($i = 0; $i < $wmuMaxFileCount; $i++) {
            if (isset($_FILES[$key]['type'][$i])) {
                $type = $_FILES[$key]['type'][$i];
                $size = $_FILES[$key]['size'][$i];
                $name = $_FILES[$key]['name'][$i];
                $path = $_FILES[$key]['tmp_name'][$i];
                $matches = wp_check_filetype($name);
                $ext = strtolower($matches['ext']);

                if (!$ext) {
                    if (preg_match('#\.(\w+)$#isu', $name, $matches)) {
                        $ext = strtolower($matches[1]);
                    }
                }

                if ($size > $this->maxFileSize) {
                    $msgArray['code'] = 'msgMaxFileSize';
                    wp_die(json_encode($msgArray));
                }

                $this->simpleMimeType($key, $ext, $typesArray, $path, $name);
            }
        }
    }

    private function simpleMimeType($key, $ext, $typesArray, $path, $name) {
        if ($key == 'wmu_images' && (!in_array($ext, $typesArray) || (in_array($ext, $typesArray) && !getimagesize($path)))) {
            $msgArray['code'] = 'msgNotAllowedFiles';
            $msgArray['typeError'] = $ext;
            wp_die(json_encode($msgArray));
        } else if (!in_array($ext, $typesArray)) {
            $msgArray['code'] = 'msgNotAllowedFiles';
            $msgArray['typeError'] = $ext;
            wp_die(json_encode($msgArray));
        }
    }

    private function getImageSizes() {
        $sizes = array();
        foreach ($this->optionsSerialized->wmuImageSizes as $_size) {
            $sizes[$_size]['width'] = get_option("{$_size}_size_w");
            $sizes[$_size]['height'] = get_option("{$_size}_size_h");
        }
        return $sizes;
    }

    public function getAttachedImages($attachIds, $currentUser = null, $size = 'full') {
        global $pagenow;
        $images = '';
        if ($attachIds) {
            $attachments = get_posts(array('include' => $attachIds, 'post__in' => $attachIds, 'post_type' => 'attachment'));
            if ($attachments && is_array($attachments)) {
                $style = '';
                if ($pagenow == 'edit-comments.php') {
                    $style .= 'max-height:100px;';
                    $style .= 'width:auto;';
                    $height = '';
                    $width = '';
                } else {
                    if (count($attachments) > 1) {
                        $width = $this->optionsSerialized->wmuImageWidth;
                        $height = $this->optionsSerialized->wmuImageHeight;
                    } else {
                        $width = $this->optionsSerialized->wmuSingleImageWidth;
                        $height = $this->optionsSerialized->wmuSingleImageHeight;
                    }

                    $style .= 'max-height:' . $height . 'px;';
                    $style .= 'height:' . $height . 'px;';
                    $style .= 'width:auto;';
                }

                foreach ($this->imageSizes as $sizeKey => $sizeValue) {
                    if (!intval($sizeValue['height']) && !intval($sizeValue['width'])) {
                        continue;
                    }

                    if ($sizeValue['height'] > 0 && $height < $sizeValue['height']) {
                        $size = $sizeKey;
                    } else if ($sizeValue['width'] > 0 && $height < $sizeValue['width']) {
                        $size = $sizeKey;
                    } else {
                        $size = 'full';
                    }
                }

                foreach ($attachments as $attachment) {
                    $deleteHtml = $this->getDeleteHtml($currentUser, $attachment, 'image');
                    $url = wp_get_attachment_url($attachment->ID);
                    $srcData = wp_get_attachment_image_src($attachment->ID, $size);
                    $src = $srcData[0];
                    $images .= "<div class='wmu-attachment wmu-attachment-$attachment->ID'>";
                    $images .= "<a id='wmu-attachemnt-link-$attachment->ID' href='$url' class='wmu-attached-image-link' target='_blank'>";
                    if ($this->optionsSerialized->wmuLazyLoadImages) {
                        $srcValue = 'data:image/png;base64,R0lGODlhAQABAAD/ACwAAAAAAQABAAACADs=';
                        $dataSrcValue = $src;
                    } else {
                        $srcValue = $src;
                        $dataSrcValue = '';
                    }
                    $images .= "<img style='$style' alt='" . esc_attr($attachment->post_excerpt) . "' title='" . esc_attr($attachment->post_excerpt) . "' id='wmu-attachemnt-$attachment->ID' class='attachment-$size size-$size wmu-attached-image' src='$srcValue' wmu-data-src='$dataSrcValue' width='$width' />";
                    $images .= "</a>";
                    $images .= $deleteHtml;
                    $images .= "</div>";
                }
            }
        }
        return $images;
    }

    public function getAttachedVideos($attachIds, $currentUser = null, $size = 'thumbnail') {
        $video = '';
        if ($attachIds) {
            $attachments = get_posts(array('include' => $attachIds, 'post__in' => $attachIds, 'post_type' => 'attachment'));
            $width = $this->optionsSerialized->wmuVideoWidth;
            $height = $this->optionsSerialized->wmuVideoHeight;
            foreach ($attachments as $attachment) {
                $deleteHtml = $this->getDeleteHtml($currentUser, $attachment, 'video');
                $url = wp_get_attachment_url($attachment->ID);
                $src = '';
                $filename = $this->getFileName($attachment);
                if (preg_match('#^video#is', $attachment->post_mime_type)) {
                    if ($this->optionsSerialized->wmuIsHtml5Video && in_array($attachment->post_mime_type, $this->html5VideoTypes)) {
                        $video .= "<div class='wmu-attachment wmu-attachment-$attachment->ID wmu-video'>";
                        $video .= "<video title='" . esc_attr($attachment->post_excerpt) . "' width='$width' controls>";
                        $video .= "<source src='$url' type='$attachment->post_mime_type'>";
                        $video .= __('Your browser does not support the video tag.', 'wpdiscuz-media-uploader');
                        $video .= "</video>";
                        $video .= $deleteHtml;
                        $video .= "</div>";
                    } else {
                        $src = plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'img' . WMUDS . 'file-icons' . WMUDS . 'video-audio-type.png');
                        $video .= "<div class='wmu-attachment wmu-attachment-$attachment->ID'>";
                        $video .= "<a href='$url' class='wmu-attached-video-link' target='_blank'>";
                        $video .= "<img class='attachment-$size size-$size wmu-attached-video' src='$src' />";
                        $video .= $filename ? " <span class='wmu-attachment-name' title='" . esc_attr($attachment->post_excerpt) . "'> $filename </span> " : '';
                        $video .= "</a>";
                        $video .= $deleteHtml;
                        $video .= "</div>";
                    }
                } else if (preg_match('#^audio#is', $attachment->post_mime_type)) {
                    if ($this->optionsSerialized->wmuIsHtml5Audio && in_array($attachment->post_mime_type, $this->html5AudioTypes)) {
                        $video .= "<div class='wmu-attachment wmu-attachment-$attachment->ID wmu-audio'>";
                        $video .= "<audio title='" . esc_attr($attachment->post_excerpt) . "' controls style='width:{$width}px'>";
                        $video .= "<source src='$url' type='$attachment->post_mime_type'>";
                        $video .= __('Your browser does not support the audio tag.', 'wpdiscuz-media-uploader');
                        $video .= "</audio>";
                        $video .= $deleteHtml;
                        $video .= "</div>";
                    } else {
                        $src = plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'img' . WMUDS . 'file-icons' . WMUDS . 'video-audio-type.png');
                        $video .= "<div class='wmu-attachment wmu-attachment-$attachment->ID'>";
                        $video .= "<a href='$url' class='wmu-attached-video-link' target='_blank'>";
                        $video .= "<img class='attachment-$size size-$size wmu-attached-video' src='$src' />";
                        $video .= $filename ? " <span class='wmu-attachment-name' title='" . esc_attr($attachment->post_excerpt) . "'> $filename </span>" : '';
                        $video .= "</a>";
                        $video .= $deleteHtml;
                        $video .= "</div>";
                    }
                }
            }
        }
        return do_shortcode($video);
    }

    public function getAttachedFiles($attachIds, $currentUser = null, $size = 'thumbnail') {
        $files = '';
        if ($attachIds) {
            $attachments = get_posts(array('include' => $attachIds, 'post__in' => $attachIds, 'post_type' => 'attachment'));
            foreach ($attachments as $attachment) {
                $deleteHtml = $this->getDeleteHtml($currentUser, $attachment, 'file');
                $url = wp_get_attachment_url($attachment->ID);
                $src = plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'img' . WMUDS . 'file-icons' . WMUDS . 'file-type-64.png');
                $filename = $this->getFileName($attachment);
                $files .= "<div class='wmu-attachment wmu-attachment-$attachment->ID'>";
                $files .= "<a href='$url' class='wmu-attached-file-link' target='_blank'>";
                $files .= "<img class='attachment-$size size-$size wmu-attached-file' src='$src' />";
                $files .= $filename ? " <span class='wmu-attachment-name' title='" . esc_attr($attachment->post_excerpt) . "'> $filename </span> " : '';
                $files .= "</a>";
                $files .= $deleteHtml;
                $files .= "</div>";
            }
        }
        return $files;
    }

    private function getDeleteHtml($currentUser, $attachment, $type) {
        $deleteHtml = "<div id='wmu-attach-remove-$attachment->ID' class='wmu-attachment-delete wmu-delete-$type' title='" . __('Delete', 'wpdiscuz-media-uploader') . "'>&nbsp;</div>";
        return $this->canEditAttachments($currentUser, $attachment) ? $deleteHtml : "<div class='wmu-separator'></div>";
    }

    public function canEditAttachments($currentUser, $attachment) {
        return current_user_can('delete_others_posts') || ($currentUser && $currentUser->ID && $currentUser->ID == $attachment->post_author);
    }

    public function getUploadButtons($uniqueId) {
        $output = '';
        if ($this->optionsSerialized->wmuIsImagesAllowed) {
            $output .= $this->getButtonsHtml('image', $uniqueId);
        }
        if ($this->optionsSerialized->wmuIsVideosAllowed) {
            $output .= $this->getButtonsHtml('video', $uniqueId);
        }
        if ($this->optionsSerialized->wmuIsFilesAllowed) {
            $output .= $this->getButtonsHtml('file', $uniqueId);
        }
        return $output;
    }

    private function getButtonsHtml($type, $uniqueId) {
        $output = '';
        if ($this->optionsSerialized->wmuIsMultiple) {
            $output .= "<label class='wmu-add wmu-multiple'>";
        } else {
            $output .= "<label class='wmu-add wmu-single'>";
            $output .= "<input id='wmu-$uniqueId-$type-0' class='wmu-$type wmu_reset wmu-hide' type='file' name='wmu_" . $type . "[]'/>";
        }
        $output .= "<span class='wpdiscuz-button wmu-icon-$type'>&nbsp;</span>";
        $output .= "<input type='hidden' class='wmu-type wmu-hide' value='$type'>";
        $output .= "</label>";
        return $output;
    }

    public function getUploadPreview($uniqueId) {
        $bool = false;
        $output = '';
        if ($this->optionsSerialized->wmuIsImagesAllowed) {
            $bool = true;
            $text = $this->optionsSerialized->wmuPhrases['wmuPhraseImageFiles'];
            $output .= $this->getPreviewHtml('image', $text, $uniqueId);
        }
        if ($this->optionsSerialized->wmuIsVideosAllowed) {
            $bool = true;
            $text = $this->optionsSerialized->wmuPhrases['wmuPhraseVideoFiles'];
            $output .= $this->getPreviewHtml('video', $text, $uniqueId);
        }
        if ($this->optionsSerialized->wmuIsFilesAllowed) {
            $bool = true;
            $text = $this->optionsSerialized->wmuPhrases['wmuPhraseOtherFiles'];
            $output .= $this->getPreviewHtml('file', $text, $uniqueId);
        }
        $output = $bool ? '<div class="wmu-action-wrap">' . $output . '</div>' : '';
        return $output;
    }

    private function getPreviewHtml($type, $text, $uniqueId) {

        if ($this->optionsSerialized->wmuIsMultiple) {
            $output = "<div class='wmu-tabs wmu-multi-file wmu-$type-tab wmu-hide'>";
            $output .= "<div class='wmu-top-title'>$text</div>";
        } else {
            $output = "<div class='wmu-tabs wmu-single-file wmu-$type-tab wmu-hide'>";
        }
        $phraseClickToAdd = $this->optionsSerialized->wmuPhrases['wmuPhraseClickToAdd'];
        for ($i = 0; $i < $this->elementCount; $i++) {
            if ($this->optionsSerialized->wmuIsMultiple) {
                $output .= "<label class='wmu-add wmu-add-$i wmu-hide wmu-show'>";
                $output .= "<span class='wmu-add-icon' title='" . $phraseClickToAdd . "'>&nbsp;</span>";
                $output .= "<input id='wmu-$uniqueId-$type-$i' class='wmu-$type wmu_reset wmu-hide' type='file' name='wmu_$type" . "[]'/>";
                $output .= "</label>";
            }
            $output .= "<div class='wmu-preview wmu-preview-$type wmu-preview-$type-$i wmu-hide'>";
            $output .= "<div class='wmu-preview-remove'>";
            if ($type == 'image') {
                $output .= "<canvas style='max-height:45px;' id='$type-$uniqueId-canvas-$i'></canvas>";
            }
            $output .= "<img class='wmu-preview-img wmu-hide' src='data:image/png;base64,R0lGODlhAQABAAD/ACwAAAAAAQABAAACADs='/>";
            $output .= "<div class='wmu-file-name'></div>";
            $output .= "<div class='wmu-delete'>&nbsp;</div>";
            $output .= "</div>";
            $output .= "</div>";
        }
        $output .= "</div>";
        return $output;
    }

    public function isUploadAllowed() {
        return $this->optionsSerialized->wmuImageTypes || $this->optionsSerialized->wmuFileTypes || $this->optionsSerialized->wmuVideoTypes;
    }

    public function canUserUpload($currentUser) {
        $bool = false;
        if ($currentUser && $currentUser->ID) {
            $userRoles = $currentUser->roles;
            foreach ($userRoles as $role) {
                if (in_array($role, $this->optionsSerialized->wmuAllowedRoles)) {
                    $bool = true;
                    break;
                }
            }
        }
        return $bool;
    }

    private function getFileName($attachment) {
        $name = false;
        if ($attachment && ($attachment->post_excerpt || $attachment->post_title)) {
            $name = $attachment->post_excerpt ? $attachment->post_excerpt : $attachment->post_title;
            if (strlen($name) > 40) {
                $name = function_exists('mb_substr') ? mb_substr($name, -40, 40, 'UTF-8') : substr($name, -40, 40);
                $name = '...' . $name;
            }
            $name = ucfirst(str_replace(array('-', '_'), ' ', $name));
        }
        return $name;
    }

    public function isImageAllowed() {
        return isset($_FILES['wmu_images']) && $_FILES['wmu_images'] && $this->optionsSerialized->wmuIsImagesAllowed;
    }

    public function isVideoAllowed() {
        return isset($_FILES['wmu_videos']) && $_FILES['wmu_videos'] && $this->optionsSerialized->wmuIsVideosAllowed;
    }

    public function isFileAllowed() {
        return isset($_FILES['wmu_files']) && $_FILES['wmu_files'] && $this->optionsSerialized->wmuIsFilesAllowed;
    }

    public function deleteLinkedAttachments($comment_id) {
        if (intval($comment_id)) {
            $imagesMeta = get_comment_meta($comment_id, self::METAKEY_IMAGE, true);
            $images = explode(',', $imagesMeta);
            if ($images && is_array($images)) {
                foreach ($images as $image) {
                    if (intval($image)) {
                        wp_delete_attachment($image);
                    }
                }
            }

            $videosMeta = get_comment_meta($comment_id, self::METAKEY_VIDEO, true);
            $videos = explode(',', $videosMeta);
            if ($videos && is_array($videos)) {
                foreach ($videos as $video) {
                    if (intval($video)) {
                        wp_delete_attachment($video);
                    }
                }
            }

            $filesMeta = get_comment_meta($comment_id, self::METAKEY_FILE, true);
            $files = explode(',', $filesMeta);
            if ($files && is_array($files)) {
                foreach ($files as $file) {
                    if (intval($file)) {
                        wp_delete_attachment($file);
                    }
                }
            }

            delete_comment_meta($comment_id, self::METAKEY_IMAGE);
            delete_comment_meta($comment_id, self::METAKEY_VIDEO);
            delete_comment_meta($comment_id, self::METAKEY_FILE);
        }
    }

    public function deleteCommentMeta($post_id) {
        global $pagenow;
        if (intval($post_id) && $pagenow && !is_wp_error($pagenow) && $pagenow == self::PAGE_EDIT_COMMENTS) {
            $postTitle = get_the_title($post_id);
            $pattern = "#comment_([\d])+_(attachment_(?:images|files|videos_audios))_[\d]+#is";
            if (trim($postTitle) && preg_match($pattern, $postTitle, $matches)) {
                if ($matches && isset($matches[1]) && isset($matches[2])) {
                    $commentId = $matches[1];
                    $metaKey = $matches[2];
                    $pattern = "#(,|^)$post_id,#is";
                    $commentMeta = get_comment_meta($commentId, $metaKey, true);
                    if ($commentMeta) {
                        $commentMeta = preg_replace($pattern, '${1}', $commentMeta);
                        update_comment_meta($commentId, $metaKey, $commentMeta);
                    }
                }
            }
        }
    }

    public function imageFixOrientation($filename) {
        $exif = exif_read_data($filename);
        if (!empty($exif['Orientation'])) {
            $image = imagecreatefromjpeg($filename);
            switch ($exif['Orientation']) {
                case 3:
                    $image = imagerotate($image, 180, 0);
                    break;
                case 6:
                    $image = imagerotate($image, -90, 0);
                    break;
                case 8:
                    $image = imagerotate($image, 90, 0);
                    break;
            }
            imagejpeg($image, $filename, 90);
        }
    }

}
