<?php
/*
 * Plugin Name: wpDiscuz - Google reCAPTCHA
 * Description: Adds No CAPTCHA on all comment forms. Stops spam and bot comments with Google reCAPTCHA
 * Version: 1.1.2
 * Author: gVectors Team (A. Chakhoyan, G. Zakaryan, H. Martirosyan)
 * Plugin URI: https://gvectors.com/product/wpdiscuz-recaptcha/
 * Author URI: https://gvectors.com/product/wpdiscuz-recaptcha/
 * Text Domain: wpdiscuz_recaptcha
 * Domain Path: /languages/
 */
require_once 'includes/gvt-api-manager.php';
require_once 'ReCaptchaOptions.php';
require_once 'src/autoload.php';

class wpDiscuzReCaptcha {

    private $reCaptcha;
    private $options;
    private $notice;
    private $pVersion;

    public function __construct() {
        new GVT_API_Manager(__FILE__, 'wpdiscuz_options_page', 'wpdiscuz_option_page');
        add_action('plugins_loaded', array(&$this, 'pluginsLoaded'), 1589);
    }

    public function validateCaptcha() {
        if ($this->options->isShowRecaptcha()) {
            $messageArray['code'] = 'wc_invalid_captcha';
            if (!isset($_POST['g-recaptcha-response']) || $_POST['g-recaptcha-response'] == '')
                wp_die(json_encode($messageArray));
            $resp = $this->reCaptcha->verify($_POST['g-recaptcha-response'], $_SERVER['REMOTE_ADDR']);
            if (!$resp->isSuccess())
                wp_die(json_encode($messageArray));
        }
    }

    public function frontendScripts($optionsSerialized) {
        if ($this->options->isShowRecaptcha()) {
            $lang = $this->options->lang ? '&hl=' . $this->options->lang : '';
            wp_register_script('wpdiscuz-google-recaptcha', 'https://www.google.com/recaptcha/api.js?onload=onloadCallback&render=explicit' . $lang, null, '1.0.0', true);
            wp_enqueue_script('wpdiscuz-google-recaptcha');
            wp_register_script('wpdiscuz-recaptcha-js', plugins_url('/js/wpdiscuz-recaptcha.js', __FILE__), array('jquery'), $this->pVersion, true);
            wp_enqueue_script('wpdiscuz-recaptcha-js');
            wp_register_style('wpdiscuz-recaptcha-css', plugins_url('/css/wpdiscuz-recaptcha.css', __FILE__), null, $this->pVersion);
            wp_enqueue_style('wpdiscuz-recaptcha-css');
            if (is_rtl()) {
                wp_register_style('wpdiscuz-recaptcha-rtl-css', plugins_url('/css/wpdiscuz-recaptcha-rtl.css', __FILE__), null, $this->pVersion);
                wp_enqueue_style('wpdiscuz-recaptcha-rtl-css');
            }
        }
    }

    public function recaptchaHtml($args, $currentUser, $uniqueId) {
        if (!$this->options->isShowRecaptcha())
            return;
        ?>
        <div class="wc-field-captcha wpdiscuz-item">
            <div class="wpdiscuz-recaptcha" id='wpdiscuz-recaptcha-<?php echo $uniqueId; ?>'></div>
            <input id='wpdiscuz-recaptcha-field-<?php echo $uniqueId; ?>' type='hidden' name='wpdiscuz_recaptcha' value='' required="required" class="wpdiscuz_reset"/>
            <div class="clearfix"></div>
        </div>
        <?php
    }

    private function initRecaptcha() {
        try {
            $requestMethod = $this->createRequestMethod();
            $this->reCaptcha = new wpDiscuzReCaptcha\ReCaptcha($this->options->getSecretKey(), $requestMethod);
        } catch (Exception $ex) {
            $this->notice = $ex->getMessage();
            add_action('admin_notices', array($this, 'adminNotices'));
        }
    }

    private function createRequestMethod() {
        if ($this->options->requestMethod != 'auto') {
            switch ($this->options->requestMethod) {
                case 'socket':
                    return new wpDiscuzReCaptcha\RequestMethod\SocketPost();
                case 'curl':
                    return new wpDiscuzReCaptcha\RequestMethod\CurlPost();
                case 'post':
                    return new wpDiscuzReCaptcha\RequestMethod\Post();
            }
        } else {
            if (function_exists('fsockopen')) {
                return new wpDiscuzReCaptcha\RequestMethod\SocketPost();
            }

            if (extension_loaded('curl')) {
                return new wpDiscuzReCaptcha\RequestMethod\CurlPost();
            }

            if (ini_get('allow_url_fopen')) {
                return new wpDiscuzReCaptcha\RequestMethod\Post();
            }
        }
    }

    public function adminNotices() {
        if (current_user_can('manage_options')) {
            echo '<div class="error">
				<p>' . __('wpDiscuz - Google Recaptcha', 'wpdiscuz_recaptcha') . ' | <span style="color:#000">' . $this->notice . '</span> | Please go to wpDiscuz Settings > Addons > Google reCAPTCHA and fill all required fields.</p>
			</div>';
        }
    }

    private function initPluginVersion() {
        if (!function_exists('get_plugins')) {
            require_once ABSPATH . 'wp-admin/includes/plugin.php';
        }
        $plugin_folder = get_plugins('/' . plugin_basename(dirname(__FILE__)));
        $plugin_file = basename(( __FILE__));
        $this->pVersion = $plugin_folder[$plugin_file]['Version'];
    }

    public function initWpDiscuzRecaptcha($args) {
        $args['wpdiscuz_recaptcha'] = 1;
        return $args;
    }

    public function pluginsLoaded() {
        if (function_exists('wpDiscuz')) {
            $this->options = new ReCaptchaOptions();
            $this->initRecaptcha();
            $this->initPluginVersion();

            add_action('wpdiscuz_front_scripts', array(&$this, 'frontendScripts'));
            add_filter('wpdiscuz_captcha_args', array(&$this, 'initWpDiscuzRecaptcha'));
            add_action('wpdiscuz_captcha_field', array(&$this, 'recaptchaHtml'), 10, 3);
            add_action('wpdiscuz_add_comment', array(&$this, 'validateCaptcha'));
            add_action('wpdiscuz_after_subscription_form', array(&$this, 'subscribtionRecaptchaHtml'));
            add_filter('wpdiscuz_before_subscription_added', array(&$this, 'validateSubscribtionCaptcha'));

            add_action('wpdiscuz_addon_tab_title', array(&$this->options, 'optionTabName'), 1);
            add_action('wpdiscuz_addon_tab_content', array(&$this->options, 'optionTabContent'), 1);
            add_action('wpdiscuz_form_init', array(&$this->options, 'initParentOptions'));
            add_action('wpdiscuz_save_options', array(&$this->options, 'saveOptions'));
            add_filter('wpdiscuz_js_options', array(&$this->options, 'addJSOption'));
            load_plugin_textdomain('wpdiscuz_recaptcha', false, dirname(plugin_basename(__FILE__)) . '/languages/');
        } else {
            add_action('admin_notices', array(&$this, 'rcRequirements'), 1);
        }
    }

    public function subscribtionRecaptchaHtml() {
        if ($this->options->isShowOnSubscribeForm) {
            ?>
            <div class="wc-field-captcha wpdiscuz-item">
                <div class="wpdiscuz-recaptcha" id='wpdiscuz-recaptcha-subscribe-form'></div>
                <input id='wpdiscuz-recaptcha-field-subscribe-form' type='hidden' name='wpdiscuz_recaptcha_subscribe_form' value='' required="required" class="wpdiscuz_reset"/>
                <div class="clearfix"></div>
            </div>
            <?php
        }
    }

    public function validateSubscribtionCaptcha($addSubscription) {
        if ($this->options->isShowRecaptcha() && $this->options->isShowOnSubscribeForm) {
            if (!isset($_POST['g-recaptcha-response']) || $_POST['g-recaptcha-response'] == '')
                return false;
            $resp = $this->reCaptcha->verify($_POST['g-recaptcha-response'], $_SERVER['REMOTE_ADDR']);
            if (!$resp->isSuccess())
                return false;
        }
        return $addSubscription;
    }

    public function rcRequirements() {
        if (current_user_can('manage_options')) {
            echo "<div class='error'><p>" . __('wpDiscuz Google reCAPTCHA requires wpDiscuz to be installed!', 'wpdiscuz_recaptcha') . "</p></div>";
        }
    }

}

$wpDiscuzReCaptcha = new wpDiscuzReCaptcha();

